function [H,acc]=convh(V,typ);
%  CONVH:  compute convex hull of points
%  Inputs:
%     V[0]          =  set of points
%     typ['numer']  = type of operation ['inter'|'outer'|'numer']
%  Outputs:
%  H[ [1 eps; 1 0] ]  = convex hull polytope
%  acc[eps] = maximum fitting error of adjacent vertices and facets
% 
% Copyright (c) SysBrain, S M Veres 19-05-2001 
 
 
if nargin<1, V=[0];end;acc=eps;H=[1 eps;1 0];
if nargin<2, typ='numer';end;
 
 
global DELTA;global KAPPA;global ERRFACTOR;global COMPLX;
 
 
if (isempty(DELTA) | isempty(KAPPA) | isempty(ERRFACTOR)), H=[];setdelta;end;
 
 
 [nv,d]=size(V);
 
 
 
 if ~isempty(COMPLX), 
    fid=fopen('complex.txt','a');
    cl=clock;
    lin=sprintf('CONVH started at %2i hrs %2i min %2.3f sec with %5i points...\n',cl(4:6),nv);
    disp(lin);
    fprintf(fid,lin);
    fclose(fid);   
 end;
 
 
 
if max(max(abs(V)))>(KAPPA+DELTA)*(1+2*d*eps), 
   disp('Message from CONVH:');
   warnkap;
   setdelta;
end;
 
 
if nv<1 | d<1, H=[];return; end;
 
 
if nv==1, 
   d2=max(d-2,0);
   if d>1, 
      H=[d 0 eps zeros(1,d2); eye(d) V';V -1];
   else 
      H=[d eps zeros(1,d2); eye(d) V';V -1];
   end;
   return;
end;
 
 
if d<2, 
   ma=max(V);mi=min(V); 
	if ma>mi+DELTA,  
	  H=[0 eps;1 ma;-1 -mi;mi -1;ma -1];
	else 
	  H=[1 eps;1 ma ];
  end;
return;
end;
 
 
  diam=0;
  for i=1:nv,
     for j=1:nv,
        di=norm(V(i,:)-V(j,:));
        if di>diam, diam=di;ci=i;cj=j;end
     end;
  end;
  ds=V(ci,:)';de=V(cj,:)';
  
  
  if diam<DELTA & typ=='numer', 
     d2=max(d-2,0);
     H=[d 0 eps zeros(1,d2); eye(d) V(1,:)';V(1,:) -1]; 
     return;
  elseif diam<ERRFACTOR*DELTA & typ=='inter', 
     d2=max(d-2,0);
     H=[d 0 eps zeros(1,d2); eye(d) V(1,:)';V(1,:) -1]; 
     return;
  end
  
  
V0=[ds';de']; % = initial set of vertices of the simplex
Vx=V-ones(nv,1)*ds'; % = relative positions of V
  
  
  lowdim=0>1;  % = value of lower dimension
  lowd=lowdim; % = 1 of lower dimension
 
 
for n=2:d,
  if ~lowd,
     A=V0(2:n,:)-ones(n-1,1)*ds'; 
     A1=eye(d)-A'*pinv(A*A')*A;  % = projection matrix
     k=1;ma=0; % initialise vertex index and max. distance
     for t=1:nv, 
       dist=Vx(t,:)*A1*Vx(t,:)'; % = distance from V0
       if ma < dist, ma=dist; k=t; end; % update
     end;
     if ma<DELTA,   % detection of lower dimensional case
        lowdim=n-1; % setting Boolean and numerical values
        lowd=1>0;
     end;
     if ~lowd, 
        V0=[V0;V(k,:)];
     end;
  end;
end;
 
 
if lowdim>0,
   
  Vp=V;na=d;          % initialisation
  Av=Vp-ones(nv,1)*Vp(1,:); % relative position of all vectors
  Av0=V0-ones(lowdim+1,1)*V0(1,:); % relativ pos. of simplex v.
  [U,S,V]=svd(Av0);   % singular value decomposition
  dl=lowdim;          % short name for lowdim
  Ac=V(:,dl+1:na)';bc=-Ac*Vp(1,:)'; % set of new equalities  
  Apr=V(:,1:dl);      % Apr = projection matrix  
  Vpr=Av*Apr;         % Vpr = projection
   
  [Cp,acs]=convh(Vpr,typ); % convex hull in projection space
   
  [Fac,nfac]=faces(Cp);      % faces
  [Vert,nvert]=vertices(Cp); % vertices
  Vert=[Vert zeros(nvert,na-dl)]*V'+ones(nvert,1)*Vp(1,:); % reverting 
  fac=[Fac(:,1:dl) zeros(nfac,na-dl)]*V'; % left side OK
  Fac=[ fac Fac(:,dl+1)+fac*Vp(1,:)'];    % reverting
   
  H=[ na-dl nfac acs zeros(1,na-2); Ac bc;Fac ; Vert -ones(nvert,1)];
  return;
end; % end of lowdim case
 
 
F1=[];                  %  initialise facet list
for i=1:d+1,
  x=V0(i,:);            % new point
  X=[V0(1:i-1,:); V0(i+1:d+1,:)]; % = all points except new one
  [a,b,ac]=planfit(X);  % fit hyperplane to X
  if ac>acc,acc=ac;end; % moinor accuracy
  sn=sign(b-a*x');      % check sign 
  F1=[F1;sn*a sn*b];    % add new facet
end;
 
P=[0 d+1 acc zeros(1,d-2);F1; V0 -ones(d+1,1)];
 
 
[Fs,nf]=faces(P);
 
 
 if typ=='inter',
   ind=max(Fs*[V -ones(nv,1)]') > ERRFACTOR*DELTA*ones(1,nv);
else
   ind=max(Fs*[V -ones(nv,1)]') > DELTA*ones(1,nv);
end;
Vv=V(ind',:); % = array of vertices outside
[nvv,d]=size(Vv); 
 
 
FV=fv(P);
 
 
while (nvv>0),
 
%  Compute the "most distant" point to be included:
 
dis=0;vin=1;                % initialisations
mascentr=mean(vertices(P)); % = mass centre 
for i=1:nvv,
     di=norm(mascentr-Vv(i,:)); % = 2-norm distance
     if di>dis, vin=i;end;  % update index into vin
end;
xn=Vv(vin,:);  % = most distant point
Vv=[Vv(1:vin-1,:);Vv(vin+1:size(Vv,1),:)]; % most distant point is omitted
 
 
[P,ac]=p_conv(xn,P,typ);
if ac>acc,acc=ac;end; % fitting accuracy is updated
 
 
 [Fs,nf]=faces(P);
if ~isempty(Vv),
  if typ=='inter',
     ind=max(Fs*[Vv -ones(nvv-1,1)]') > ERRFACTOR*DELTA*ones(1,nvv-1);
  else
     ind=max(Fs*[Vv -ones(nvv-1,1)]') > DELTA*ones(1,nvv-1);
  end;
  Vv=Vv(ind',:);   % = array of vertices outside
end;
 
nvv=size(Vv,1);  % update number of vertices outside
 
end; % end of adding the new point xn to the convex hull
 
 
H=P;
 
 
 if ~isempty(COMPLX), 
    fid=fopen('complex.txt','a');
    cl=clock;
    lin=sprintf('  CONVH finished %2i hrs %2i min %2.3f sec.\n',cl(4:6));
    disp(lin);
    fprintf(fid,lin);
    fclose(fid);   
 end;
 
 
return;
 
