/*
 * sfun_ad.c: Level 2 S-function for
 *
 * Author:  Narve Ekrol, Data Respons AS
 * Updated: 21.11.00
 * Version: 1.0
 *
 * Revisions: 21.04.01 Karl-Petter Lindegaard:
 *               - Number of states set to zero.
 *               - Alternative RPM measurement based on measured position. It
 *                 is worth emphasizing that since this velocity measurement is
 *                 based on the difference between two position measurement
 *                 divided by the time interval between them, a "hack" compensating
 *                 for QNXs lower sample rate during the first 1000 samples has
 *                 been included.
 * 			  29.03.06 Asbjrn Hamre Aga and Jonas Tyss
 *				 - Modified for CyberRigI
 *               - Uncommented overheat detection  
 */

// Undefine this macro to use "original" velocity measurement
#define ALTERNATIVE_VELOCITY_MEASUREMENT

#define DEBUG_MODE 0
#define S_FUNCTION_NAME sfun_motor
#define S_FUNCTION_LEVEL 2

#define CONTSTATES 0 /* Number of continuous states */
#define NMOTORS 8

#include <errno.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <sys/neutrino.h>

#include "OpalPrint.h"
#include "tmwtypes.h"
#include "simstruc.h"
#include "motorDrv.h"
#define Input(element) (*uPtrs[element])  /* Pointer to Input Port0 */


static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumSFcnParams(S, 0);  // Number of expected parameters
    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S))
	{
        return; // Parameter mismatch will be reported by Simulink
    }

    ssSetNumContStates(S, CONTSTATES);
    ssSetNumDiscStates(S, 0);
    if (!ssSetNumInputPorts(S, 1)) return;

    ssSetInputPortWidth(S, 0, NMOTORS);
    ssSetInputPortDirectFeedThrough(S, 0, 0);

    if (!ssSetNumOutputPorts(S, 1)) return;

    ssSetOutputPortWidth(S, 0, NMOTORS);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    // Take care when specifying exception free code - see sfuntmpl.doc
    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

#define MDL_INITIALIZE_CONDITIONS
static void mdlInitializeConditions (SimStruct *S)
{
  // Initialize card.
  // Reset
  ThreadCtl(_NTO_TCTL_IO,0); // rights to io-operations
  SelectCard(0);
  Init4I27();
}

#define MDL_UPDATE
static void mdlUpdate(SimStruct *S, int_T tid)
{
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
  InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0); //fetches the values supplied to the s-function in Simulink
  real_T  *y = ssGetOutputPortRealSignal(S, 0);

  int i,dir, motor;
  long cmd;

  static long PrevCmd[NMOTORS],NumSampleCmd[NMOTORS];
  static time_t OverheatTime;
  time_t curTime;

  int  iHeatStatus;

#ifdef ALTERNATIVE_VELOCITY_MEASUREMENT
  static double PrevPos[] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 }; //extended from 4 to 8
  static time_T lastTime = -1.0;
  static short numMajorSteps = 0;
  double deltaT, adouble;
  unsigned short isMajorTimeStep;
#endif


	/////////////////////////////////////////////////
    // Check for overheat
  //SelectCard(0);
  //iHeatStatus = (in8(BASE_ADDRESS +1) >> 2) & 0x3 ; // Select bit 2 and 3

  /*SelectCard(1);
  iHeatStatus = iHeatStatus & ( (in8(BASE_ADDRESS +1) >> 2) & 0x3); // Select bit 2 and 3
	*/
/*
  if (iHeatStatus == 3) // ok
  {
	  time(&OverheatTime);
	  y[2] = 1;
  }
  else
  {
	  time(&curTime);
	  y[2]=0;
	  if ((curTime - OverheatTime)> 10)
	  {
		  Init4I27(); // Stop motor if overheated for 10 seconds
		  return;
	  }
  }*/

//calculates delta_T
#ifdef ALTERNATIVE_VELOCITY_MEASUREMENT
  if (isMajorTimeStep = ssIsMajorTimeStep(S))
  {
    deltaT = ssGetT(S) - lastTime;
    lastTime = ssGetT(S);
    if (numMajorSteps < 1000)
		numMajorSteps++;
  }
#endif


  ////////////////////////////////////////////////
  // Read command and set new reference if needed
  SelectCard(0);  // Select first card
  for (i=0;i<NMOTORS; i++)
  {
		motor  = i;

		if (4>i && i>1) // Select motor on the second card, index is 1
		{
			SelectCard(1);
			motor = i-2;
		}
        else if (6>i && i>3)
        {
            SelectCard(2);
            motor = i-4;
        }
        else if (i>5)
        {
            SelectCard(3);
            motor = i-6;
        }
        

		// Read velocity
#ifndef ALTERNATIVE_VELOCITY_MEASUREMENT
		y[i] = ReadVelocityRPM(motor);
#else
		////////////////////////////////////////////////////
		// Alternative velocity measurement
        if (isMajorTimeStep)
		{
			adouble = ReadPosition(motor);
			y[i] = ((adouble-PrevPos[i])*60.0)/deltaT;
			//if (i==0)
			//	y[i] = -y[i];
			PrevPos[i] = adouble;

			// QNX (for some reason) runs the first 1000 samples at a lower rate.
			// It is assumed that this rate is 1/3 of the expected rate and hence
			// the velocities (calculated using positions) must be divided by 3.
            if (numMajorSteps < 1000)
                y[i] = y[i]/3.0;

		}
#endif

		// does not need decimal precision on rpm.
		// It is done to make a valid comparision with last command
		cmd = (long) Input(i);

		// Change dir on motor 1 to get cmd > 0 to be forward
		if (motor==0)
		{
			cmd *= -1;
			y[i] *= -1;
		}

		// In case the motor reaches the en stop we have to force a new command
		// Cannot do this to fast -> have to wait 40 samples -> 2 sek
		if ((NumSampleCmd[i]>40) &&
			(abs(y[i]-PrevCmd[i])>abs(PrevCmd[i]/2)) ) // 40 makes 2 sek
		{
			PrevCmd[i]=0;
		}

        if (NumSampleCmd[i]<50) NumSampleCmd[i]++;

		if (PrevCmd[i] != cmd)
		{
			NumSampleCmd[i]= 0;
			PrevCmd[i] = cmd;

			if (cmd < 0){ // Negative RPM
				dir = -1;
				cmd *= -1;
			}
			else
				dir = 1;

			if (DEBUG_MODE) OpalPrint("Motor %d RPM: %f\n",i,(float)y[i]);

			//LoadVelocity(i,cmd,dir);
			if (DEBUG_MODE) OpalPrint("Motor %d Cmd %d Dir %d\n",motor,cmd,dir);


			LoadVelocityRPM(motor,(double)cmd,dir);
			WriteCommand(motor,SttCmd); //writes to the output ports of the card, starts motion

		}// end if != PrevCmd

  } // end for motors
}

#define MDL_DERIVATIVES
#if defined(MDL_DERIVATIVES)
static void  mdlDerivatives(SimStruct *S)
{
}
#endif

#define MDL_TERMINATE
#if defined (MDL_TERMINATE)
static void mdlTerminate(SimStruct *S)
{
    int i;
	// Reset all cards
    for(i=0;i<NCARDS;i++)
    {
        SelectCard(i);
        Reset4I27();
    }
}
#endif

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif


















