/*
 * sfun_stepmotor.c: Level 2 S-function for CyberRig
 *
 * Author:  Narve Ekrol, Data Respons AS
 * Updated: 03.11.00
 * Updated: 20.01.03 Bjrnar Vik: Porting to QNX 6.2
 * Updated: 20.02.04 Hkon H. Solberg
 * Updated: 27.04.06 Jonas Tyss and Absjrn Hamre Aga 
 *              - Modified for CyberRig I
 *              - Modified to turn HOFF on upon reset
 * Version: 1.1
 */
#define DEBUG_MODE 0
#define S_FUNCTION_NAME sfun_stepmotor
#define S_FUNCTION_LEVEL 2

#define CONTSTATES 0 /* Number of continuous states */
#define NAXIS 3
#define NCARDS 3

#define BASE_S1 0x300      // The card's I/O base addresses
#define BASE_S2 0x310
#define BASE_S3 0x320

#define AXIS1 0x04
#define STARTSPEED_LSB 0x7f // 0x012f works but does not sound good
#define STARTSPEED_MSB 0x00

#define SLEWSPEED_LSB 0x7f
#define SLEWSPEED_MSB 0x00

#define UP    0x40 // Rudder move right
#define DOWN  0x48 // Rudder move left

// Values for Start/Stop function
#define START 0x30  // Output pulse is started, stop on
#define STOP 0x08   // Output pulse is stopped
#define RESET 0x08  // Reset the controller, same cmd as stop

#define START_HIGH  0x10  // Run constant speed at Starting Speed Register until stop command is given
#define START_LOW   0x11  // Run constant speed at Slew Speed Register until stop command is given

#include <errno.h>
#include <hw/inout.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <sys/neutrino.h>

#include "OpalPrint.h"

#include "tmwtypes.h"

#include "simstruc.h"

#define Input(element) (*uPtrs[element])  /* Pointer to Input Port0 */

static int BASE_ADDRESS = BASE_S1;

static void SelectCard(int i)
{
    switch (i)
    {
        case 0:
            BASE_ADDRESS = BASE_S1;
            break;
        case 1:
            BASE_ADDRESS = BASE_S2;
            break;
        case 2:
            BASE_ADDRESS = BASE_S3;
            break;
        default:
            BASE_ADDRESS = BASE_S2;
            OpalPrint("No such card, base adress set to BASE_S2 ");
    }
}

static void mdlInitializeSizes(SimStruct *S)
{
    ssSetNumSFcnParams(S, 0);  /* Number of expected parameters */

    if (ssGetNumSFcnParams(S) != ssGetSFcnParamsCount(S)) {
        return; /* Parameter mismatch will be reported by Simulink */
    }

    ssSetNumContStates(S, CONTSTATES);
    ssSetNumDiscStates(S, 0);
    if (!ssSetNumInputPorts(S, 1)) return;
    ssSetInputPortWidth(S, 0, NAXIS*NCARDS);
    ssSetInputPortDirectFeedThrough(S, 0, 0);

    if (!ssSetNumOutputPorts(S, 1)) return;
    ssSetOutputPortWidth(S, 0, 1);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);

    /* Take care when specifying exception free code - see sfuntmpl.doc */
    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
}

static void mdlInitializeSampleTimes(SimStruct *S)
{
    ssSetSampleTime(S, 0, INHERITED_SAMPLE_TIME);
    ssSetOffsetTime(S, 0, 0.0);
}

#define MDL_INITIALIZE_CONDITIONS
static void mdlInitializeConditions (SimStruct *S)
{
  real_T *x0 = ssGetContStates(S);
  int i,j;

  ThreadCtl(_NTO_TCTL_IO,0); // rights to io-operations

  for (i=0;i<CONTSTATES;i++)
  {
		x0[i] = 0.0;
  }
  for (i=0;i<NCARDS;i++)
  {
      SelectCard(i);
      // Initialization of step motor card
      for (j=0; j<NAXIS; j++)
      {
          // Set STARTING speed (ref. constant speed)
          out8(BASE_ADDRESS + 4*(j+1), 0x81); // Address speed reg
          out8(BASE_ADDRESS + 1+ 4*(j+1), STARTSPEED_LSB); // Write LSB
          out8(BASE_ADDRESS + 2+ 4*(j+1), STARTSPEED_MSB); // Write MSB
          
          // Set SLEW speed (ref. constant speed)
          // We dont use this but set it anyway in case someone should try
          out8(BASE_ADDRESS + 4*(j+1), 0x82); // Address speed reg
          out8(BASE_ADDRESS + 1+ 4*(j+1), SLEWSPEED_LSB); // Write LSB
          out8(BASE_ADDRESS + 2+ 4*(j+1), SLEWSPEED_MSB); // Write MSB
          
          // Reset / stop motors
          out8(BASE_ADDRESS + AXIS1*(j+1), RESET);   // 0x08 Stop/reset
      }
      
      // Set GP to control motor current / motor stop
      // pin 4 (GP 1-3) should be set high to get low motor current on the step motor
      // and pin 5 (GP 4-6) should be set low (HOFF) to get the motor running
      // The GP 1-6 are inverted, this is probably configurable
      out8(BASE_ADDRESS   , 0x0); // Select adr 0
      out8(BASE_ADDRESS+1 , 0x4); // write enable
      
      // Set some (one) of the C0, C1 or C2 bits high
      // NB! bit 0 and 1 is used to select bit, the value is in the 3 bit
      out8(BASE_ADDRESS   , 0x7); // Select adr 7
      
      out8(BASE_ADDRESS+1 , 0x0); // Set GP 1
      out8(BASE_ADDRESS+1 , 0x1); // Set GP 2
      out8(BASE_ADDRESS+1 , 0x2); // Set GP 3
      
      // Set HOFF bit low, NB these bits are inverted
      // NB! bit 0 and 1 is used to select bit, the value is in the 3 bit
      out8(BASE_ADDRESS   , 0x8); // Select adr 8
      
      out8(BASE_ADDRESS+1 , 0x4); // Set GP 4  // NB inverted !!!
      out8(BASE_ADDRESS+1 , 0x5); // Set GP 5
      out8(BASE_ADDRESS+1 , 0x6); // Set GP 6
      
      out8(BASE_ADDRESS   , 0x0); // Select adr 0
      out8(BASE_ADDRESS+1 , 0x0); // write enable
  } 
  
  if (DEBUG_MODE)
      OpalPrint("init ok\n");
}

#define MDL_UPDATE
static void mdlUpdate(SimStruct *S, int_T tid)
{
}
static void mdlOutputs(SimStruct *S, int_T tid)
{
	int i,j;

	int cmd = -2;

	int iStepCnt = 0;
	int iDir = 1;
	short bCnt0, bCnt1;
	InputRealPtrsType  uPtrs;

	uPtrs = ssGetInputPortRealSignalPtrs(S,0);
    
    for(i=0;i<(NCARDS*NAXIS);i++)
    {
        cmd =  (int) Input(i);

        if (3>i)
        {
            SelectCard(0);
            j=i;
        }
		else if (6>i && i>2) // Select motor on the second card, index is 1
		{
			SelectCard(1);
            j=i-3;
		}
        else if (9>i && i>5)
        {
            SelectCard(2);
            j=i-6;
        }
          
        if (cmd == 0)
        {
            out8(BASE_ADDRESS + AXIS1*(j+1), RESET);   // 0x08 Stop/reset
        }
        else
        {
            
            if (cmd >0)
            {
                iDir = -1;      // Gives increase in volt
                iStepCnt = cmd; // make positive cnt;
            }
            else
            {
                iDir = 1;       // Gives a decrease in volt
                iStepCnt = -cmd;// make positive cnt;
            }
            
            bCnt0 = iStepCnt & 0x00ff ; // Find 8 lsbit
            bCnt1 = (iStepCnt >> 8) & 0x00ff;
            
            if (DEBUG_MODE)
                OpalPrint("Axis  %d Cmd %d Dir %d iStepCnt %d\n",i, cmd,iDir,iStepCnt,bCnt1,bCnt0);
            
            // Reset
            out8(BASE_ADDRESS + AXIS1*(j+1), RESET);        // 0x08; Reset
            
            // Operation mode
            if (iDir == 1) // select direction
                out8(BASE_ADDRESS + AXIS1*(j+1), 0x44);		// operation mode + dir
            else
                out8(BASE_ADDRESS + AXIS1*(j+1), 0x4C);	    // operation mode - dir
            
            // Set counter
            out8(BASE_ADDRESS + 4*(j+1), 0x80); // Address speed reg
            out8(BASE_ADDRESS + 1 + 4*(j+1), bCnt0); // Write LSB
            out8(BASE_ADDRESS + 2 + 4*(j+1), bCnt1); // Write
            out8(BASE_ADDRESS + 3 + 4*(j+1), 0x00); // Write MSB
            
            // Start
            out8(BASE_ADDRESS + AXIS1*(j+1), START_LOW);	// 0x10;
        }// end if cmd==0 else
    }// end for axis*cards
}

#define MDL_DERIVATIVES
#if defined(MDL_DERIVATIVES)
static void mdlDerivatives(SimStruct *S)
{
}
#endif

#define MDL_TERMINATE
#if defined (MDL_TERMINATE)
static void mdlTerminate(SimStruct *S)
{
	int i,j;
    for(i=0;i<NCARDS;i++)
    {
        SelectCard(i);
        for (j=0;j<NAXIS; j++)
        {
            out8(BASE_ADDRESS + AXIS1*(j+1), RESET);   // 0x08 Stop/reset
        }

        out8(BASE_ADDRESS   , 0x0); // Select adr 0
        out8(BASE_ADDRESS+1 , 0x4); // write enable (set)
        out8(BASE_ADDRESS   , 0x8); // Select adr 8
        out8(BASE_ADDRESS+1 , 0x0); // clear GP 4  // NB inverted !!!
        out8(BASE_ADDRESS+1 , 0x1); // clear GP 5
        out8(BASE_ADDRESS+1 , 0x2); // clear GP 6 NOT USED
        out8(BASE_ADDRESS   , 0x0); // Select adr 0
        out8(BASE_ADDRESS+1 , 0x0); // write enable (clear)
    }

	if (DEBUG_MODE) OpalPrint("Step motor stopped!\n");
}
#endif

#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif
