function [v, inc, neighborvert]=lqrcVertices(poly)
% SYNTAX: v=lqrcVertices(poly)
%
% Returns the vertices of a bounded polyhedron specifed by inequalities
% Numerical computations are performed using CDD
% http://www.ifor.math.ethz.ch/~fukuda/cdd_home/cdd.html
%
%   input:  poly - polyhedron (where each row (a1 a2 ... an b)
%                  defines an inequality a^T x <= b)
%		
%   outputs: v -   each column of v is a vertex. If the polyhedron is
%                  not bounded, an empty matrix v is returned.
% 
%   Authors:   Tor A Johansen, Idar Petersen 
%   Copyright: SINTEF Electronics and Cybernetics
%   History:   000426
%
%   Updated: Petter Tndel, May 2003
%   Now uses mex gateway to call CDD
%
%
%   Comments from cddmex.txt:
% CDDLIB matlab interface (December 4, 2002)
% 
% This is an MATLAB interface (cddmex.c) to cddlib, created by
% 
% Mato Baotic <baotic@aut.ee.ethz.ch>, and
% Torrisi Fabio <torrisi@aut.ee.ethz.ch>.
% 
% The help file (cddmex.m) was written by
% Alberto Bemporad <bemporad@unisi.it>
% 
% Notes by Fabio:
% To compile the Mexfile is enough to run from the MATLAB prompt
% mex -v -ICDDDIR/lib-src cddmex.c CDDDIR/lib-src/libcdd.a
% 
% where CDDDIR is the directory where cddlib is installed.
% 
% To compile under windows one needs to obtain and install gnumex <http://www.mrc-cbu.cam.ac.uk/Imaging/gnumex20.html>.
% 
% -------
% readme file written by Komei Fukuda <fukuda@cs.mcgill.ca>

% H = struct('A', poly(:,1:end-1), 'B', poly(:,end));
% V=cddmex('extreme',H);
% v = V.V';
% if size(V.R,1)
%     warning('Extreme rays in vertice enumeration')
% end
% return

if nargout >= 2 %Computing incidence
   incidence = 1;
else
   incidence = 0;
end

numofhyperplanes = size(poly,1);
n=size(poly,2)-1;

% Prepare data for CDD
[fid,msg] = fopen('lqrc8682.ine','w');
if (fid == -1)
   error(msg);
end
[rows,cols] = size(poly);
cr = setstr(13);
fprintf(fid,'H-representation%s\n',cr);
fprintf(fid,'begin%s\n',cr);
fprintf(fid,'%d %d real%s\n',rows,cols,cr);
for i=1:rows
   if rem(poly(i,cols),1)==0
      fprintf(fid,'%.0f ',poly(i,cols));
   else
      fprintf(fid,'%.12f ',poly(i,cols));
   end
   for j=1:cols-1
      if rem(poly(i,j),1)==0
         fprintf(fid,'%.0f ',-poly(i,j));
      else
         fprintf(fid,'%.12f ',-poly(i,j));      
      end
   end
   fprintf(fid,'%s\n',cr);
end
fprintf(fid,'end%s\n',cr);
if incidence
   fprintf(fid,'incidence%s\n',cr);
end
fprintf(fid,'dynout_off%s\n',cr);
fprintf(fid,'stdout_off%s\n',cr);
fclose(fid);

% Call CDD  
%!C:\Jobb\Work\mompqp\lqrcCDD.exe lqrc8682.ine >lqrc8682.txt
!Z:\DIPLOM\mpc\sektor\lqrcCDD.exe lqrc8682.ine >lqrc8682.txt
%!C:\"Documents and Settings"\Irene\Desktop\DIPLOM\mpc\sektor\lqrcCDD.exe lqrc8682.ine >lqrc8682.txt
% Read the result from CDD
fid = fopen('lqrc8682.ext','r');

line = fgets(fid);
while ~strcmp(line,['begin' setstr(13) setstr(10)])
   line = fgets(fid);
end

dim = fscanf(fid,'%f');
line = fgets(fid);
poly1 = fscanf(fid,'%f');
rows = dim(1);
cols = dim(2);

v = zeros(cols-1,rows);
for i=1:rows
   if poly1(1+(i-1)*cols) ~= 1
      rows =rows-1;
      %disp('Warning, unbounded polyhedron')
            v = [];
            return
   else
      v(:,i) = poly1(2+(i-1)*cols:i*cols);   
   end
end
v=v(:,1:rows);
fclose(fid);

if incidence
   % Read the result from CDD
   fid = fopen('lqrc8682.ecd','r');
   
   line = fgets(fid);
   while ~strcmp(line,['begin' setstr(13) setstr(10)])
      line = fgets(fid);
   end
   
   line = fgets(fid);
   data = sscanf(line,'%f');
   numv = data(1);
   numh = data(2);
   
   %%%%%
   numv = size(v,2);
   
   inc=[];
   for i=1:numv
      line=fgets(fid);
      numhplanes = sscanf(line,'%f');
      start = findstr(line,':');
      data = sscanf(line(start+1:length(line)),'%f')';
      if numhplanes<=0
         inc{i} = setdiff([1:numofhyperplanes],data);
      else
         inc{i} = data;
      end
   end
   fclose(fid);
   
   maxinc=0;
   for j=1:length(inc)
      maxinc = max(maxinc,length(inc{j}));
   end
   allinc=[];
   for j=1:length(inc)
      inc1=zeros(1,maxinc+1);
      inc1(1,1:length(inc{j}))=inc{j};
      allinc=[allinc;inc1];
   end
   inc=allinc;
   
   if 0
   neighborvert = neighborvertices(poly,v,inc);
   neighborvert = zeros(size(inc,1),size(inc,2)+2);
   
   for j1=1:size(v,2)
      inc1=inc(j1,find(inc(j1,:)~=0));
      for j2=j1+1:size(v,2)
         inc2=inc(j2,find(inc(j2,:)~=0));
         common_hplanes=intersct(inc1,inc2);
         if length(common_hplanes)>=n-1
            A = poly(common_hplanes,1:n);
            if rank(A,1e-8)>=n-1 %Neighboring vertices
               firstzero = find(neighborvert(j1,:)==0);
               if isempty(firstzero)
                  neighborvert=[neighborvert zeros(size(neighborvert,1),1)];
                  firstzero = size(neighborvert,2);
               else
                  firstzero = firstzero(1);
               end
               neighborvert(j1,firstzero) = j2;
               
               firstzero = find(neighborvert(j2,:)==0);
               if isempty(firstzero)
                  neighborvert=[neighborvert zeros(size(neighborvert,1),1)];
                  firstzero = size(neighborvert,2);
               else
                  firstzero = firstzero(1);
               end
               neighborvert(j2,firstzero) = j1;
            end
         end
      end
   end
	end

   delete('lqrc8682.ecd')
end

delete('lqrc8682.txt')
delete('lqrc8682.ine')
delete('lqrc8682.ext')