function [h1,fitacc]=hadjust(h,P,typ);
%  HADJUST: adjust a new hyperplane to a polytope
%  Inputs:
%    h   =  hyperplane inequality
%    P   =  a polytope
%     typ['numer']  = type of operation
%  Outputs:
%    h1  =  adjusted hyperplane
%    fitacc = fitting accuracy of adjusted point
% 
% Copyright (c) SysBrain, S M Veres 19-05-2001 
 
%  Find global numerical bounds:
 
global DELTA;global KAPPA;global ERRFACTOR;
if nargin<3, typ='numer';end;
 
%  Activate interface if any of the bounds is undefined:
 
if (isempty(DELTA) | isempty(KAPPA)), setdelta;P=[];end;
 
%  Transpose `h' if necessary:
 
if size(h,2)==1, h=h';end;
 
%  Find  dimension of space:
 
d=size(h,2)-1;
 
%  Normalise hyperplane inequality
 
nrm=norm(h(1:d));
if nrm>d*DELTA, 
   h=h/nrm;
else 
  error('Uncertain subspace used in HADJUST.M');
end;
 
%  Diagnostics:
 
if isempty(P), 
   h1=h;fitacc=eps;
   return;
end;
 
%  Get list of vertices:
 
[V,nv]=vertices(P);
 
%  Find the vertices which fit to the new hyperplane
%  with DELTA-accuracy:
 
B=abs([V -ones(nv,1)]*h')<DELTA;fitacc=eps;
Vv=V(B,:); % Vv = list of DELTA-near vertices
 
%   Exit if there is no DELTA-near vertex:
 
if isempty(Vv), h1=h;return;end;
 
nv1=size(Vv,1);
 
if nv1>0 & typ=='inter',
   nv2=nv1;
   while nv2>0,
      h1=[h1(1:d) h1(d+1)-ERRFACTOR*DELTA];
      B=abs([V -ones(nv,1)]*h1')<DELTA; 
      Vv=V(B,:); % Vv = list of DELTA-near vertices
      nv2=size(Vv,1);
   end;
   return;
elseif nv1>0 & typ=='outer',
   nv2=nv1;
   while nv2>0,
      h1=[h1(1:d) h1(d+1)+ERRFACTOR*DELTA];
      B=abs([V -ones(nv,1)]*h1')<DELTA; 
      Vv=V(B,:); % Vv = list of DELTA-near vertices
      nv2=size(Vv,1);
   end;
   return;
end;
 
%  Deal with the case of 1 DELTA-near point:
 
if nv1==1, 
   c=h(1:d)*Vv(1,1:d)';h1=[h(1:d) c];return;
end;
 
%  Deal with the 2D case:
 
if d==2 & nv1>=2,
   v=Vv(1,:)-Vv(2,:);
   [U,S,L]=svd(v);
   ov=L(:,2);
   c=mean(Vv(1:2,:))*ov;
   if sign(h(3))==sign(c),
       h1=[ov' c];
   else
       h1=-[ov' c];
   end;
   fitacc=eps;
   return;
end;
 
%  Deal with the case of  `d'  DELTA-near vertices:
 
if nv1==d,
    
    V0=Vv(2:d,:)-ones(d-1,1)*Vv(1,:);
 
%  Extract a basis from the linear manifold spanned by `V0':
 
   [U,S,L]=svd(V0); 
   inds=abs(S)>eps;
   rnk=sum(sum(inds));      % L(:,1:rnk)' is the basis
 
%  Take the orthogonal complement of the basis:
 
   Lc=L(:,rnk+1:d)';   % = orthog. complement of basis
   coord=Lc*h(1:d)';   % co-ordinates of `h1' in orth. compl.
   h2=coord'*Lc;        % projection of h1 onto orth. compl. = h2
   h1=h2/norm(h2);     % note that norm(h2) = norm(coord);
   c=mean(Vv*h1');     % positioning of the new hyperplane
   h1=[h1 c];          % definition  of new hyperplane
   fitacc=max(abs([Vv -ones(d,1)]*h1')); % fitting accuracy
   Vvc=V(~B,:);        % rest of polytope vertices 
   nvc=size(Vvc,1);Vvc=[Vvc -ones(nvc,1)];
   
%  Test if rest of vertices are DELTA-far :
 
   if nvc==0, return;end;
   if min(abs(Vvc*h1'))>DELTA, return; end; 
   
end;
 
 
%  Fit the facet to d number of DELTA-near vertices:
 
[F,nf]=faces(P); 
BvF=[Vv -ones(nv1,1)]*F'<DELTA;indx=(1:nf); 
nF=sum(BvF);[maxnF,maxi]=max(nF);
if maxnF>=d, 
   h1=h/norm(h(1:d));
   if sign(h1(d+1))==sign(F(maxi,d+1)),
   h1=F(maxi,:);fitacc=eps;
   else
   h1=-F(maxi,:);fitacc=eps;    
   end;
return;
end;
 
%  Find the largest low-dimensional simplex among DELTA-near points:
 
[V1,B1]=findsimp(Vv);L1=size(V1,1);
B2=B;B2(B)=B1;
V0=V1(2:L1,:)-ones(L1-1,1)*V1(1,:);
 
%  Extract a basis from the linear manifold spanned by `V0':
 
[U,S,L]=svd(V0);
inds=abs(S)>eps;
rnk=sum(sum(inds));      % L(:,1:rnk)' is the basis
 
%  Take the orthogonal complement of the basis:
 
Lc=L(:,rnk+1:d)';   % = orthog. complement of basis
coord0=Lc*h(1:d)';   % co-ordinates of `h1' in orth. compl.
h2=coord0'*Lc;       % projection of h1 onto orth. compl. = h2
h1=h2/norm(h2);     % note that norm(h2) = norm(coord);
c=mean(V1*h1');     % positioning of the new hyperplane
h1=[h1 c];          % definition of new hyperplane
 
Vvc=V(~B2,:);        % rest of polytope vertices 
nvc=size(Vvc,1);Vvc=[Vvc -ones(nvc,1)];
Dfar=min(abs(Vvc*h1')); % distance of the rest of vertices
 
%  Test if rest of vertices are DELTA-far :
 
if  Dfar>DELTA, return; end; 
    
%  Rotate `h1' until Dfar<DELTA :
 
rcount=0;dd=length(coord0);R=eye(dd);
 
while Dfar<DELTA,
 
rcount=rcount+1;
k=mod(rcount,dd);if k==0,k=1;end;
m=mod(k+1,dd);if m==0,m=1;end;
E=eye(dd);
E(k,k)=1-DELTA;E(m,m)=1-DELTA;
E(k,m)=DELTA;E(m,k)=-DELTA;
R=R*E;
coord=R*coord0;    
h2=coord'*Lc        % projection of h1 onto orth. compl. = h2
h1=h2/norm(h2);     % note that norm(h2) = norm(coord);
c=mean(V1*h1');     % positioning of the new hyperplane
h1=[h1 c];          % definition of new hyperplane
fitacc=max(abs([V1 -ones(nv1,1)]*h1')); % fitting acccuracy    
Dfar=min(abs(Vvc*h1')); % distance of the rest of vertices
 
if rcount>100, 
    disp('   ');
    disp('Numerically extra hard problem encountered in HADJUST...  ');
    disp('Switch to the use of  typ="inter" or typ="outer"');
    disp('for the type of operation in FCONVH, UPDATE or CONVH, P_CONV ,');
    disp('depending on whether inner or outer bounding is');
    disp('satisfactory in your application.');
    error('Cannot continue without "typ" being redefined...'); 
end;
 
end;
 
%  Give warning if fitting error exceeds global bound:
 
if fitacc>DELTA, warndel;
disp('(...message from HADJUST at end of routine)');end;
return;
 
 
 
 
