/*
 * motorDrv.h: Level 2 S-function for
 *
 * Author:  Narve Ekrol, Data Respons AS
 * Updated: 21.11.00
 * Version: 1.0
 *
 * Revisions: 21.04.01 Karl-Petter Lindegaard:
 *               - Corrected controller gains: Original ones => internal instability.
 *               - Corrected the function ReadDataWord() due to bugs.
 *               - Added function for reading shaft position ReadPosition().
 *            20.01.03 Bjrnar Vik
 *               - Ported to QNX 6.2. conio.h replaced by hw/inout.h. Functions outp() and outw() replaced
 *                 by out8() and out16() respectively and inp() and inpw() replaced by in8() and inp16() respectively.
 *            29.05.06 Jonas Tyss and Asbjrn Hamre Aga
 *               - Modified to support CyberRig I
 *
 * NOTE: The transmission of the bow-thruster is not one to one! This must be
 *       accounted for, otherwise desired and measured RPM related to the motor
 *       and not the propeller shaft! On the other hand, this is just a constant
 *       factor that will be found during the forthcoming system identification.
 */


#define CARDMOTORS 2
#define BASE1 0x200
#define BASE2 0x210
#define BASE3 0x240
#define BASE4 0x250
#define NCARDS 4


// Constants
#define KP 100 	// 100
#define KI 0 	// 100
#define KD 0 	// 3000
#define IL 10
#define ACCELERATION 1000	// Max value 1073741823 but it would kill the power supply

#define MOTOR_CMD_POFF 0x04

#define CMD_PORT_BOTH 0x0C
#define DATA_PORT_BOTH 0x0D

#define LOAD_PARAMS_CMD 	0x1F
#define START_MOTION_CMD 	0x01

#define STOP_ABRUPTLY 	0x20
#define STOP_SMOOTH 	0x40
#define TURN_OFF_MOTOR 	0x10
#define POSITION

// Commands
#define ResetData      0x84  //                       { to look for a valid reset }
#define ResetDataMask  0xBF  //

// Trajectory cons
#define VELOCITY_MODE 0x0800
#define ENCODER_LINES 500
#define CLOCK_F 8000000.0f

//{ 'write' commands, no data }
#define ResetCmd           0x00                      //{ reset the LM629 }
#define SttCmd             0x01                      //{ start motion }
#define DfhCmd             0x02                      //{ define home }
#define SipCmd             0x03                      //{ set index position }
#define UdfCmd             0x04                      //{ update filter }
#define Port8Cmd           0x05                      //{ select 8 bit output (pwm) }
#define Port12Cmd          0x06                      //{ select 12 bit output (better not !) }

//{ 'write' commands, all followed by data }
#define LPESCmd            0x1A       // stop on error }
#define LPEICmd            0x1B       // interrupt on error }
#define MskICmd            0x1C       // mask interrupts }
#define RstICmd            0x1D       // reset interrupts }
#define LFilCmd            0x1E       // load filter parameters }
#define LTrjCmd            0x1F       // load trajectory }
#define SBpACmd            0x20       // set breakpoint absolute }
#define SBpRCmd            0x21       // set breakpoint relative }

// Read commands
#define RddvCmd				0x07      // read desired velocity }
#define RddpCmd			    0x08      // read desired position }
#define RdipCmd				0x09      // read index position }
#define RdrpCmd			    0x0A      // read real position }
#define RdrvCmd			    0x0B      // read real velocity }
#define RdsigsCmd			0x0C      // read signals register }
#define RdsumCmd			0x0D      // read integration }

//{ status byte masks (same as low byte of signals register except bit 0) }
#define MotorOffMask       0x80
#define BreakPointMask     0x40
#define ExcessivePosMask   0x20
#define WraparoundMask     0x10
#define IndexPulseMask     0x08
#define TrajectoryComMask  0x04
#define CommandErrorMask   0x02
#define LM629BusyMask      0x01

//{ signals register masks }
#define HostInterruptMask  0x8000
#define AccelLoadedMask    0x4000
#define UDFExecutedMask    0x2000
#define ForwardDirMask     0x1000
#define VelocityModeMask   0x0800
#define OnTargetMask       0x0400
#define TurnOffOnEPEMask   0x0200
#define EightBitModeMask   0x0100

//{ next six bits can use status register masks }
#define AcqNextIdxMask     0x0001

//{ load trajectory command (LTRJ) control bits }
#define ForwardDirection   0x1000
#define VelocityMode       0x0800
#define PositionMode       0x0000
#define StopSmoothlyCmd    0x0400
#define StopAbruptlyCmd    0x0200
#define TurnOffMotorCmd    0x0100
#define TurnOnMotorCmd     0x0000
#define LoadAccBit         0x0020
#define AccRelBit          0x0010
#define LoadVelBit         0x0008
#define VelRelBit          0x0004
#define LoadPosBit         0x0002
#define PosRelBit          0x0001
#define LoadAllTraj        LoadAccBit | LoadVelBit | LoadPosBit

// load filter command (LFIL) control bits }
#define LoadKPBit          0x08
#define LoadKIBit		   0x04
#define LoadKDBit          0x02
#define LoadILBit          0x01
#define LoadAllFil         LoadKPBit | LoadKIBit | LoadKDBit | LoadILBit

#include <stdio.h>
#include <time.h>

#include <errno.h>
#include <hw/inout.h>
#include <math.h>
#include <string.h>


// Global variables
static int SamplingInterval = 0;
static int MotionMode = VELOCITY_MODE;
static int BASE_ADDRESS = BASE1;


static void SelectCard(int i)
{
    switch (i)
    {
        case 0:
            BASE_ADDRESS = BASE1;
            break;
        case 1:
            BASE_ADDRESS = BASE2;
            break;
        case 2:
            BASE_ADDRESS = BASE3;
            break;
        case 3:
            BASE_ADDRESS = BASE4;
            break;
        default:
            BASE_ADDRESS = BASE1;
            OpalPrint("No such card, base adress set to BASE1 ");
    }
}

// Check busy bit
short LM629Busy(int motor)
{
	short busy = ( (in8(BASE_ADDRESS + MOTOR_CMD_POFF *(1+motor) ) & 0x1) != 0); //(!= 0)
	return busy;
}

// Writes command to control register
void WriteCommand(int motor, int cmd)
{
	while (LM629Busy(motor))  {}    // active wait

	out8(BASE_ADDRESS + MOTOR_CMD_POFF *(1+motor), cmd);
}

// Writes command to data register
void WriteDataWord(int motor, int data)
{
	int hi, lo,cnt;

	data = data & 0xffff; // only a word

	lo = data & 0x00ff;
	hi = (data & 0xff00) >> 8;

  // Have to wait a bit before checking busy flag
	for (cnt=0;cnt<10;cnt++){}

	while (LM629Busy(motor)) 	{	}//active wait

	out8(BASE_ADDRESS + MOTOR_CMD_POFF *(1+motor) + 1, hi);  // 21.01.03 BVIK: Removed return variable since return of out8() is void
	out8(BASE_ADDRESS + MOTOR_CMD_POFF *(1+motor) + 1, lo);  // 21.01.03 BVIK: Removed return variable since return of out8() is void

}

// Read word from data register
unsigned short ReadDataWord(int motor)
{
	unsigned short w;
	while (LM629Busy(motor)) 	{	}//active wait

	w  = in8(BASE_ADDRESS + MOTOR_CMD_POFF *(1+motor) + 1);
	w  = (w << 8);
	w += in8(BASE_ADDRESS + MOTOR_CMD_POFF *(1+motor) + 1);

	return w;
}

// Read long from data register
long ReadLongint(int motor)
{
	long l  = (long) ReadDataWord(motor);
	l = (l << 16);
	l += (long)ReadDataWord(motor);

	return l;
}

// Write long to data register
void WriteLongint(int motor,long data)
{
	int hiw, low;

	low = (int)  (data & 0x0000ffff);
	hiw = (int) ((data >> 16) & 0xffff);

	WriteDataWord(motor, hiw);
	WriteDataWord(motor, low);
}

static void  EnableHBridge(int motor)
{
	short portsave, enablemask;

	enablemask = 0xff - (motor+1);  // Enable bit is 0x1 and 0x2 for motor 0 and 1
	portsave = in8(BASE_ADDRESS); // Port A
	portsave = portsave & enablemask;
	out8(BASE_ADDRESS,portsave);
}

static void  DisableHBridge(int motor)
{
	short portsave, enablemask;

	enablemask = motor+1;  // Enable bit is 0x1 and 0x2 for motor 0 and 1
	portsave = in8(BASE_ADDRESS); // Port A
	portsave = portsave | enablemask;
	out8(BASE_ADDRESS,portsave);
}

int ReadLM629Status(int motor)
{
	int LM629BugDelay = 0;
  	int status = in8(BASE_ADDRESS + MOTOR_CMD_POFF *(1+motor) );

  	while (LM629BugDelay < 1000)	LM629BugDelay++;
	// This nonsense is because of a NS LM629 chip bug -- If you poll the status
	// register too fast, the LM629 crashes. This seems to be a feature on newer
	// (DC >89 or so) chips

	return status;
}

static short Reset4I27()
{
	int i;
	short resetD;

	// Init PPI Write to control port
	out8(BASE_ADDRESS + 0x03, 0x8b); 				// OutAInBC    { set to normal directions }
	out8(BASE_ADDRESS + 0x00, (0x0|0x1|0x2) ); 		// IdlePPIA or Motor0Disable or Motor1Disable. Writes to the 82C55 PORT A;

	resetD = in8(BASE_ADDRESS);

	if (resetD != 3){
		if (DEBUG_MODE) OpalPrint("Read from Areg after reset: %x Should be %x\n\n",resetD,(0x0|0x1|0x2));
	}
	if (DEBUG_MODE) OpalPrint("Reset cmd\n");

  	for (i=0;i<CARDMOTORS;i++)
		WriteCommand(i,ResetCmd); // Reset

	for (i=0;i<20000;i++) {}// wait for a goodly length of time

	for (i=0;i<CARDMOTORS;i++)
	{
		resetD = in8(BASE_ADDRESS+MOTOR_CMD_POFF*(i+1) ) &  ResetDataMask;    //0xBF
		if (resetD !=  ResetData)
		{
			if (DEBUG_MODE) OpalPrint("\nReset of LM629 failed!!, Resetdata is %x\n",resetD);
			return 0;
		}
	}
	if (DEBUG_MODE) OpalPrint("Reset OK! Resetdata is %x\n",resetD);
	return 1;
}


void static LoadVelocity(int motor, long CntPrSample, int dir)
{
	WriteCommand( motor,	LTrjCmd);				//  LTrjCmd	: load trajectory

   if (dir >0)
		WriteDataWord(motor,  ForwardDirection | MotionMode | LoadVelBit);  // LoadVelBit
   else
		WriteDataWord(motor,  MotionMode | LoadVelBit);  // LoadVelBit

	WriteLongint(motor, CntPrSample);
}

void static LoadVelocityRPM(int motor, double rpm,int dir)
{
	long countsPerSample;
	double samplePeriod = 0.0;

	samplePeriod = (SamplingInterval+1.0)/ (CLOCK_F / 2048.0);
	countsPerSample = (long)(samplePeriod * ENCODER_LINES * (4.0/60.0) * rpm * 65536.0);
	// X 4 because of four transitions per line on encoder, /60 for rpM, X 65536 to make 16 bit number with 16 bit fraction }


   if (DEBUG_MODE) OpalPrint("Motor %d CPS %d Dir %d \n",motor,countsPerSample,dir);
   LoadVelocity(motor, countsPerSample,dir);


}
double static ReadVelocityRPM(int motor)
{
	long countsPerSample;
	double samplePeriod = 0.0, rpm=0.0;

	WriteCommand(motor,RdrvCmd);
	countsPerSample = ReadLongint(motor);

	// 16 least significant bits not used for RdrvCmd.
	// countsPerSample = countsPerSample & 0xFFFF0000;

	samplePeriod = (SamplingInterval+1.0)/ (CLOCK_F / 2048.0);

	rpm = ((double)countsPerSample) / (samplePeriod * ENCODER_LINES * (4.0/60.0) * 65536.0);

	return rpm;
}

static double ReadPosition(int motor)
{
	long counts;
	double samplePeriod, n;

	WriteCommand(motor, RdrpCmd);
	counts = ReadLongint(motor);

	n = ((double)counts) / (ENCODER_LINES * 4.0);

	return n;

}
void LoadFilter(int motor)
{
 	// Set filter 0
 	int filtcontword;
  	WriteCommand(motor, LFilCmd);  	// load filter param

  	filtcontword = SamplingInterval |  LoadAllFil; // SamplingInterval | 0xf all filter bit
  	WriteDataWord(motor,filtcontword);

  	WriteDataWord(motor,KP);					//	KP);
  	WriteDataWord(motor,KI);					//	KI);
  	WriteDataWord(motor,KD);					//	KD);
  	WriteDataWord(motor,IL);					//  IL);

  	WriteCommand(motor, UdfCmd); // Update filter
}

void TurnOffMotor(int motor)
{
  WriteCommand(motor,  LTrjCmd);			//  LTrjCmd	: load trajectory
  WriteDataWord(motor, TurnOffMotorCmd);	//	TurnOffMotorCmd
  WriteCommand(motor,  SttCmd);				//  SttCmd  : Start motion
}

void SetAccFact(int motor,long accFact)
{
  WriteCommand(motor,	LTrjCmd);				//  LTrjCmd	: load trajectory
  WriteDataWord(motor,MotionMode| LoadAccBit); // 0x0020 -> LoadAccBit);
  WriteLongint(motor,accFact);						// 1000 -> acceleration);
}

void Init4I27()
{
	int i,j, status;
	SamplingInterval = 0; //

	// Reset all cards
	for (j=0;j<NCARDS;j++)
    {
        SelectCard(j);
        Reset4I27();
		
		for (i=0;i<CARDMOTORS;i++)
		{
			WriteCommand(i,Port8Cmd);			// Sets 8 bit output port. Indicates PWM
			LoadFilter(i);						// Load filter
			TurnOffMotor(i);					// Turn off to start motion
			SetAccFact(i,ACCELERATION); 		// Set acc factor

			WriteCommand(i, LPESCmd);			// LPESCmd -> Stop on error need this?? no
			WriteDataWord(i, 0xffff);			// The motor will stop at this cnt.  Set to be 0xffff
			WriteCommand(i,DfhCmd);				// DfhCmd  -> define home


			 EnableHBridge(i);
		}
    }
}





